@@ -470,7 +470,7 @@ void agc_free(struct agc *s)
 
 void agc_valueparse(struct agc *s, char *param)
    {
-   char *save, *key, *value;
+   char *save = NULL, *key, *value;
 
    key = strtok_r(param, "=", &save);
    value = strtok_r(NULL, "=", &save);
