@@ -17,6 +17,7 @@
 #   If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "../config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -69,7 +70,12 @@ static int jack_process_callback(jack_nframes_t n_frames, void *arg)
 
 static void custom_jack_error_callback(const char *message)
    {
-   fprintf(stderr, "jack: %s\n", message);
+   fprintf(stderr, "jack error: %s\n", message);
+   }
+
+static void custom_jack_info_callback(const char *message)
+   {
+   fprintf(stderr, "jack info: %s\n", message);
    }
 
 static void jack_shutdown_callback()
@@ -102,6 +108,9 @@ struct audio_feed *audio_feed_init(struct threads_info *ti)
       }
    self->threads_info = ti;
    jack_set_error_function(custom_jack_error_callback);
+#ifdef HAVE_JACK_SET_INFO_FUNCTION
+   jack_set_info_function(custom_jack_info_callback);
+#endif
 
    if (!(self->jack_h = jack_client_open(self->jack_client_name, JackUseExactName | JackServerName, NULL, getenv("IDJC_JACK_SERVER"))))
       {
