@@ -211,6 +211,7 @@ static void avcodecdecode_play(struct xlplayer *xlplayer)
 int avcodecdecode_reg(struct xlplayer *xlplayer)
    {
    struct avcodecdecode_vars *self;
+   int error;
    
    pthread_once(&once_control, once_init);
    if (!(xlplayer->dec_data = self = calloc(1, sizeof (struct avcodecdecode_vars))))
@@ -267,9 +268,9 @@ int avcodecdecode_reg(struct xlplayer *xlplayer)
       }
    pthread_mutex_unlock(&mutex);
     
-   posix_memalign((void *)&self->outbuf, 64, AVCODEC_MAX_AUDIO_FRAME_SIZE);
+   error = posix_memalign((void *)&self->outbuf, 64, AVCODEC_MAX_AUDIO_FRAME_SIZE);
    self->floatsamples = malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE * 2);
-   if (!self->outbuf || !self->floatsamples)
+   if (error || !self->floatsamples)
       {
       fprintf(stderr, "avcodecdecode_reg: malloc failure\n");
       avcodecdecode_eject(xlplayer);
