@@ -1,6 +1,6 @@
 /*
 #   encoder.c: the encoder framework for the streaming module of idjc
-#   Copyright (C) 2007 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2007-2009 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -92,6 +92,13 @@ static int encoder_get_resample_mode(char *rm_string)
 
 static void encoder_free_input_ringbuffers(struct encoder *self)
    {
+   struct timespec ms10 = { 0, 10000000 };
+   
+   if (self->jack_dataflow_control == JD_ON)
+      self->jack_dataflow_control = JD_FLUSH;
+   while (self->jack_dataflow_control != JD_OFF)
+      nanosleep(&ms10, NULL);
+   
    if (self->input_rb[0])
       jack_ringbuffer_free(self->input_rb[0]);
    if (self->input_rb[1])
@@ -99,31 +106,34 @@ static void encoder_free_input_ringbuffers(struct encoder *self)
    self->input_rb[0] = self->input_rb[1] = NULL;
    }
 
-static int encoder_unlink(struct encoder *self)
+static void encoder_free_resampler(struct encoder *self)
+   {
+   int i;
+   
+   for (i = 0; i < 2; i++)
+      if (self->src_state[i])
+         {
+         src_delete(self->src_state[i]);
+         self->src_state[i] = NULL;
+         }
+   }
+
+static void encoder_plugin_terminate(struct encoder *self)
    {
    struct timespec ms10 = { 0, 10000000 };
-   int i;
-
-   if (!self->run_request_f)
-      {
-      fprintf(stderr, "encoder_unlink: run_request_f was unset when this function was called\n");
-      return FAILED;
-      }
-   self->run_request_f = FALSE;		/* tell the encoder to make an orderly exit */
-   fprintf(stderr, "encoder_unlink: waiting for encoder to finish\n");
+      
+   self->run_request_f = FALSE;
+   if (self->encoder_state != ES_STOPPED)
+      fprintf(stderr, "encoder_plugin_terminate: waiting for encoder to finish\n");
    while (self->encoder_state != ES_STOPPED)
       nanosleep(&ms10, NULL);
-   if (self->jack_dataflow_control != JD_OFF)
-      {
-      self->jack_dataflow_control = JD_FLUSH;
-      while(self->jack_dataflow_control == JD_FLUSH)
-         nanosleep(&ms10, NULL);
-      }
+   }
+
+static void encoder_unlink(struct encoder *self)
+   {
+   encoder_plugin_terminate(self);
    encoder_free_input_ringbuffers(self);
-   if (self->resample_f)
-      for (i = 0; i < self->n_channels; i++)
-         src_delete(self->src_state[i]);
-   return SUCCEEDED;
+   encoder_free_resampler(self);
    }
 
 static long encoder_input_rb_mono_downmix(jack_ringbuffer_t **rb, float *bptr, int max_samples)
@@ -567,19 +577,7 @@ int encoder_start(struct threads_info *ti, struct universal_vars *uv, void *othe
       return SUCCEEDED;
       }
    failed:
-   self->run_request_f = FALSE;
-   while (self->encoder_state != ES_STOPPED)
-      nanosleep(&ms10, NULL);
-   if (self->jack_dataflow_control != JD_OFF)
-      {
-      self->jack_dataflow_control = JD_FLUSH;
-      while (self->jack_dataflow_control != JD_OFF)
-         nanosleep(&ms10, NULL);
-      }
-   encoder_free_input_ringbuffers(self);
-   if (self->resample_f)
-      for (i = 0; i < self->n_channels; i++)
-         src_delete(self->src_state[i]);
+   encoder_unlink(self);
    fprintf(stderr, "encoder_start: failed to start the encoder\n");
    return FAILED;
    }
@@ -588,11 +586,7 @@ int encoder_stop(struct threads_info *ti, struct universal_vars *uv, void *other
    {
    struct encoder *self = ti->encoder[uv->tab];
    
-   if (!encoder_unlink(self))
-      {
-      fprintf(stderr, "encoder_stop: failed to stop encoder\n");
-      return FAILED;
-      }
+   encoder_unlink(self);
    if (self->output_chain)
       fprintf(stderr, "encoder_stop: function has been called with encoder_op objects still attached\n");
    fprintf(stderr, "encoder_stop: encoder is stopped\n");
@@ -603,10 +597,8 @@ int encoder_update(struct threads_info *ti, struct universal_vars *uv, void *oth
    {
    struct encoder *self = ti->encoder[uv->tab];
    
-   if (!encoder_unlink(self))
-      return FAILED;
+   encoder_unlink(self);
    return encoder_start(ti, uv, other);
-   return SUCCEEDED;
    }
  
 int encoder_new_metadata(struct threads_info *ti, struct universal_vars *uv, void *other)
@@ -739,9 +731,5 @@ void encoder_destroy(struct encoder *self)
       free(self->metaformat);
    if (self->metaformat_mp3)
       free(self->metaformat_mp3);
-   if (self->input_rb[0])
-      jack_ringbuffer_free(self->input_rb[0]);
-   if (self->input_rb[1])
-      jack_ringbuffer_free(self->input_rb[1]);
    free(self);
    }
