@@ -227,16 +227,18 @@ char *dol, *dor, *dil, *dir;
 char *oggpathname, *sndfilepathname, *avformatpathname, *speexpathname, *speextaglist, *speexcreatedby;
 char *playerpathname, *seek_s, *size, *playerplaylist, *loop, *resamplequality;
 char *agc_param, *fade_mode;
+char *rg_db;
 
 /* dictionary look-up type thing used by the parse routine */
 struct kvpdict kvpdict[] = {
-	 { "PLRP", &playerpathname },		/* The media-file pathname for playback */
-         { "SEEK", &seek_s },			/* Playback initial seek time in seconds */
-         { "SIZE", &size },			/* Size of the file in seconds */
-         { "PLPL", &playerplaylist },		/* A playlist for the media players */
-         { "LOOP", &loop },			/* play in a loop */
-         { "MIXR", &mixer_string },		/* Control strings */
-         { "COMP", &compressor_string },	/* packed full of data */
+         { "PLRP", &playerpathname },   /* The media-file pathname for playback */
+         { "RGDB", &rg_db },            /* Replay Gain volume level controlled at the player end */
+         { "SEEK", &seek_s },           /* Playback initial seek time in seconds */
+         { "SIZE", &size },             /* Size of the file in seconds */
+         { "PLPL", &playerplaylist },   /* A playlist for the media players */
+         { "LOOP", &loop },             /* play in a loop */
+         { "MIXR", &mixer_string },     /* Control strings */
+         { "COMP", &compressor_string },/* packed full of data */
          { "GATE", &gate_string },
          { "MICS", &microphone_string },
          { "NORM", &normalizer_string },
@@ -1823,7 +1825,7 @@ int main(int argc, char **argv)
    int i, new_left_pause, new_right_pause;
    static int old_lmic_on = 0, old_rmic_on = 0, old_aux_on = 0;
    jack_nframes_t nframes;
-   char *artist = NULL, *title = NULL;
+   char *artist = NULL, *title = NULL, *replaygain = NULL;
    double length;
    int sync = FALSE;
    int use_dsp;
@@ -2058,25 +2060,25 @@ int main(int argc, char **argv)
 
       if (!strcmp(action, "playleft"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_l, playerpathname, atoi(seek_s), atoi(size)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_l, playerpathname, atoi(seek_s), atoi(size), atof(rg_db)));
          fflush(stdout);
          }
       if (!strcmp(action, "playright"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_r, playerpathname, atoi(seek_s), atoi(size)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_r, playerpathname, atoi(seek_s), atoi(size), atof(rg_db)));
          fflush(stdout);
          }
       if (!strcmp(action, "playnoflushleft"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_l, playerpathname, atoi(seek_s), atoi(size)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_l, playerpathname, atoi(seek_s), atoi(size), atof(rg_db)));
          fflush(stdout);
          }
       if (!strcmp(action, "playnoflushright"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_r, playerpathname, atoi(seek_s), atoi(size)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_r, playerpathname, atoi(seek_s), atoi(size), atof(rg_db)));
          fflush(stdout);
          }
-      
+    
       if (!strcmp(action, "playmanyjingles"))
          {
          fprintf(stdout, "context_id=%d\n", xlplayer_playmany(plr_j, playerplaylist, loop[0]=='1'));
@@ -2120,9 +2122,9 @@ int main(int argc, char **argv)
       
       if (!strcmp(action, "ogginforequest"))
          {
-         if (oggdecode_get_metainfo(oggpathname, &artist, &title, &length))
+         if (oggdecode_get_metainfo(oggpathname, &artist, &title, &length, &replaygain))
             {
-            fprintf(stdout, "OIR:ARTIST=%s\nOIR:TITLE=%s\nOIR:LENGTH=%f\nOIR:end\n", artist, title, length);
+            fprintf(stdout, "OIR:ARTIST=%s\nOIR:TITLE=%s\nOIR:LENGTH=%f\nOIR:REPLAYGAIN_TRACK_GAIN=%s\nOIR:end\n", artist, title, length, replaygain);
             fflush(stdout);
             }
          else
