@@ -17,11 +17,13 @@
 #   If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "gnusource.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
 #include "kvpdict.h"
+#include "bsdcompat.h"
 
 /* kvp_extract_value: extract the value of a key value pair from a string. The value is a copy of the orignial and is allocated on the heap.  The returned "value" should be destroyed with free() when no longer needed.  The string supplied is truncated at the = sign */
 char *kvp_extract_value(char *pair)
