@@ -17,11 +17,12 @@
 #   If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define _GNU_SOURCE
+#include "gnusource.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "kvpparse.h"
+#include "bsdcompat.h"
 
 int kvp_parse(struct kvpdict *kvpdict, FILE *fp)
    {
