@@ -18,7 +18,6 @@
 */
 
 #include "../config.h"
-#ifdef HAVE_LAME
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -145,6 +144,7 @@ static void live_mp3_encoder_main(struct encoder *encoder)
          free(s->mp3buf);
          goto bailout;
          }
+
       ++encoder->oggserial;
       s->packetflags = PF_INITIAL;
       s->lame_samples = 0;
@@ -238,4 +238,3 @@ int live_mp3_encoder_init(struct encoder *encoder, struct encoder_vars *ev)
    return SUCCEEDED;
    }
 
-#endif /* HAVE_LAME */
