@@ -18,9 +18,12 @@
 */
 
 #include "../config.h"
-#ifdef HAVE_LAME
 
+#ifdef DYN_LAME
+#include "lame.h"
+#else
 #include <lame/lame.h>
+#endif /* DYN_LAME */
 #include "sourceclient.h"
 
 struct lme_data
@@ -39,4 +42,3 @@ struct lme_data
 
 int live_mp3_encoder_init(struct encoder *encoder, struct encoder_vars *ev);
 
-#endif /* HAVE_LAME */
