@@ -104,7 +104,7 @@ static void live_ogg_build_metadata(struct encoder *encoder, struct loe_data *s)
    fprintf(stderr, "live_ogg_build_metadata: metadata for encoder %d\nartist=%s\ntitle=%s\n", encoder->numeric_id, s->artist, s->title);
    }
 
-static int live_ogg_write_packet(struct encoder *encoder, ogg_page *op, int flags)
+int live_ogg_write_packet(struct encoder *encoder, ogg_page *op, int flags)
    {
    struct encoder_op_packet packet;
    char *buffer;
