@@ -22,6 +22,7 @@
 #include "gnusource.h"
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 #include <jack/jack.h>
 #ifdef DYN_MAD
 #include "mad.h"
@@ -38,14 +39,27 @@
 #define REJECTED 0
 
 #define BSIZ 16384
-#define MAD_SCALE ((float)(1L << MAD_F_SCALEBITS))
+#define MAD_SCALE ((float)(1L << (MAD_F_SCALEBITS - 18)))
 
 static int metadata_form[4] = { DM_JOINED_L1, DM_JOINED_UC, DM_JOINED_UCBE, DM_JOINED_U8 };
 
-static inline float scale(mad_fixed_t sample)
+static inline float scale(struct xlplayer *xlplayer, mad_fixed_t sample)
    {
-   return (float)sample / MAD_SCALE;
-   }
+   float ret;
+   
+   if (xlplayer->dither)
+      {
+      ret = floorf((float)sample / (float)(1L << (MAD_F_SCALEBITS - 18)));
+   
+      if (rand_r(&xlplayer->seed) < (RAND_MAX >> 1))
+         ret -= 1.0f;
+      if (rand_r(&xlplayer->seed) > (RAND_MAX >> 1))
+         ret += 1.0f;
+      return ret / 262144.0f;
+      }
+   else
+      return sample / (float)(1L << MAD_F_SCALEBITS);
+   }   
 
 static int mp3decode_get_frame(struct xlplayer *xlplayer)
    {
@@ -243,7 +257,7 @@ static void mp3decode_play(struct xlplayer *xlplayer)
    struct mad_pcm *pcm;
    jack_default_audio_sample_t *lp, *rp, *dp, gain;
    mad_fixed_t *left_ch, *right_ch, *lc, *rc;
-   int nchannels, nsamples, frame_code, pcmlength, toskip;
+   int nchannels, nsamples, frame_code, pcmlength, toskip, delay;
    SRC_DATA *src_data = &(xlplayer->src_data);
    struct chapter *chapter;
 
@@ -315,9 +329,9 @@ static void mp3decode_play(struct xlplayer *xlplayer)
          src_data->data_out = realloc(src_data->data_out, src_data->output_frames * nchannels * sizeof (float));
          for (nsamples = pcmlength; nsamples; nsamples--)
             {
-            *dp++ = scale(*(lc++));
+            *dp++ = scale(xlplayer, *(lc++));
             if (nchannels == 2)
-               *dp++ = scale(*(rc++));
+               *dp++ = scale(xlplayer, *(rc++));
             }
          if (src_process(xlplayer->src_state, src_data))
             {
@@ -348,9 +362,9 @@ static void mp3decode_play(struct xlplayer *xlplayer)
          while (nsamples--)
             {
             gain = xlplayer_get_next_gain(xlplayer);
-            *lp++ = gain * scale(*(lc++));
+            *lp++ = gain * scale(xlplayer, *(lc++));
             if (nchannels == 2)
-               *rp++ = gain * scale(*(rc++));
+               *rp++ = gain * scale(xlplayer, *(rc++));
             }
          if (nchannels == 1)
             memcpy(xlplayer->rightbuffer, xlplayer->leftbuffer, xlplayer->op_buffersize);
@@ -370,11 +384,12 @@ static void mp3decode_play(struct xlplayer *xlplayer)
    else
       xlplayer->playmode = PM_EJECTING;
 
-   chapter = mp3_tag_chapter_scan(&self->taginfo, xlplayer->play_progress_ms);
+   delay = xlplayer_calc_rbdelay(xlplayer);
+   chapter = mp3_tag_chapter_scan(&self->taginfo, xlplayer->play_progress_ms + delay);
    if (chapter && chapter != self->current_chapter)
       {
       self->current_chapter = chapter;
-      xlplayer_set_dynamic_metadata(xlplayer, metadata_form[chapter->encoding], "", "", chapter->text, 0);
+      xlplayer_set_dynamic_metadata(xlplayer, metadata_form[chapter->encoding], "", "", chapter->text, delay);
       }
    }
 
