@@ -1,6 +1,6 @@
 /*
 #   oggdec.c: ogg file parser for xlplayer
-#   Copyright (C) 2008 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2008-2009 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -81,11 +81,57 @@ static int vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to get AR
    vorbis_info vi;
    vorbis_comment vc;
    int samplerate;
-   char *tmp;
 
    vorbis_info_init(&vi);
    vorbis_comment_init(&vc);
 
+   void obtain_tag_info(char *name, char **target, int multiple)
+      {
+      int tags = vorbis_comment_query_count(&vc, name);
+      int size, i;
+
+      if (tags == 0)
+         {
+         *target = strdup("");
+         return;
+         }
+
+      if (tags == 1)
+         {
+         *target = strdup(vorbis_comment_query(&vc, name, 0));
+         return;
+         }
+
+      if (multiple)
+         {
+         /* calculate the space needed */
+         size = tags;
+         for (i = 0; i < tags; i++)
+            size += strlen(vorbis_comment_query(&vc, name, i));
+
+         if (!(*target = malloc(size)))
+            {
+            *target = strdup("");
+            fprintf(stderr, "vorbis_get_samplerate: malloc failure\n");
+            return;
+            }
+         *target[0] = '\0';
+
+         /* collect a slash separated list of tags */
+         for (i = 0; i < tags; i++)
+            {
+            strcat(*target, vorbis_comment_query(&vc, name, i));
+            if (i < tags - 1)
+               strcat(*target, "/");
+            }
+         }
+      else
+         {
+         /* grab the last comment when only a single will do */
+         *target = strdup(vorbis_comment_query(&vc, name, tags - 1));
+         }
+      }
+
    /* enforce that first header yields the sample rate, that granule pos for the header is zero and */
    /* that the third and final header finishes on a page boundary */
    if (oggdec_get_next_packet(self) && vorbis_synthesis_headerin(&vi, &vc, &self->op) >= 0 && vi.rate &&
@@ -97,16 +143,9 @@ static int vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to get AR
       samplerate = self->samplerate[self->ix] = vi.rate;
       self->channels[self->ix] = vi.channels;
       
-      /* Upper case tag names are the standard form but lower case tags also exist in the wild */
-      if ((tmp = vorbis_comment_query(&vc, "ARTIST", 0)) || (tmp = vorbis_comment_query(&vc, "artist", 0)))
-         self->artist[self->ix] = strdup(tmp);
-      if ((tmp = vorbis_comment_query(&vc, "TITLE", 0))  || (tmp = vorbis_comment_query(&vc, "title", 0)))
-         self->title[self->ix] = strdup(tmp);
-      
-      if (self->artist[self->ix] == NULL)
-         self->artist[self->ix] = strdup("");
-      if (self->title[self->ix] == NULL)
-         self->title[self->ix] = strdup("");
+      obtain_tag_info("artist", &self->artist[self->ix], TRUE);
+      obtain_tag_info("title", &self->title[self->ix], TRUE);
+      obtain_tag_info("replaygain_track_gain", &self->replaygain[self->ix], FALSE);
       }
    else
       {
@@ -236,7 +275,7 @@ static void oggflac_metadata_callback(const FLAC__StreamDecoder *decoder, const
    
    int match(char *t, char *comment)
       {
-      return !strncmp(t, comment, strlen(t));
+      return !strncasecmp(t, comment, strlen(t));
       }
    
    char *end(char *t)
@@ -247,17 +286,23 @@ static void oggflac_metadata_callback(const FLAC__StreamDecoder *decoder, const
       return t;
       }
 
-   void copy_tag(char *t, char **target)
+   void copy_tag(char *t, char **target, int multiple)
       {
       int j;
+      char *old, *new;
       
       for (j = 0; j < vc->num_comments; j++)
          {
          if (match(t, (char *)vc->comments[j].entry))
             {
-            if (*target)
-               free(*target);
-            *target = strdup(end((char *)vc->comments[j].entry));
+            old = strdup(*target);
+            new = end((char *)vc->comments[j].entry);
+            *target = realloc(*target, strlen(old) + strlen(new) + 2);
+            if (old[0] && multiple)
+               sprintf(*target, "%s/%s", old, new);
+            else
+               strcpy(*target, new);
+            free(old);
             }
          }
       if (*target == NULL)
@@ -282,10 +327,9 @@ static void oggflac_metadata_callback(const FLAC__StreamDecoder *decoder, const
          for (i = 0; i < vc->num_comments; i++)
             fprintf(stderr, "%s\n", vc->comments[i].entry);
 
-         copy_tag("ARTIST=", &self->artist[self->ix]);
-         copy_tag("artist=", &self->artist[self->ix]);
-         copy_tag("TITLE=", &self->title[self->ix]);
-         copy_tag("title=", &self->title[self->ix]);
+         copy_tag("artist=", &self->artist[self->ix], TRUE);
+         copy_tag("title=", &self->title[self->ix], TRUE);
+         copy_tag("replaygain_track_gain=", &self->replaygain[self->ix], FALSE);
          }
       else
          fprintf(stderr, "oggflac_metadata_callback: unhandled FLAC metadata type\n");
@@ -359,24 +403,50 @@ static void speex_get_comments(struct oggdec_vars *self, char *comments, int len
    char *c=comments;
    int len, i, nb_fields;
    char *end;
-   
+
+   void handle_keyval(char *key, char *val, char **target, char *match, int multiple)
+      {
+      char *old;
+
+      if (!strcasecmp(key, match))
+         {
+         if (*target && *target[0] && multiple)
+            {
+            old = strdup(*target);
+            *target = realloc(*target, strlen(old) + strlen(val) + 2);
+            sprintf(*target, "%s/%s", old, val);
+            free(old);
+            }
+         else
+            {
+            *target = realloc(*target, strlen(val) + 1);
+            strcpy(*target, val);
+            }
+         }
+      }
+  
    void handle_comment(char *comment, int comment_length)
       {
-      if (comment_length > 7 && !strncmp("author=", comment, 7))
+      
+      char *key = malloc(comment_length + 1);
+      char *val;
+      
+      memcpy(key, comment, comment_length);
+      key[comment_length] = '\0';
+      val = strchr(key, '=');
+      if (!val)
          {
-         self->artist[self->ix] = realloc(self->artist[self->ix], comment_length - 6);
-         memcpy(self->artist[self->ix], comment + 7, comment_length - 7);
-         self->artist[self->ix][comment_length - 7] = '\0';
+         fprintf(stderr, "Invalid/corrupted comments\n");
+         free(key);
          return;
          }
+      *val++ = '\0';
 
-      if (comment_length > 6 && !strncmp("title=", comment, 6))
-         {
-         self->title[self->ix] = realloc(self->title[self->ix], comment_length - 5);
-         memcpy(self->title[self->ix], comment + 6, comment_length - 6);
-         self->title[self->ix][comment_length - 6] = '\0';
-         return;
-         }
+      handle_keyval(key, val, &self->artist[self->ix], "author", TRUE);
+      handle_keyval(key, val, &self->artist[self->ix], "artist", TRUE);
+      handle_keyval(key, val, &self->title[self->ix], "title", TRUE);
+      handle_keyval(key, val, &self->replaygain[self->ix], "replaygain_track_gain", FALSE);
+      free(key);
       }
    
    self->artist[self->ix] = NULL;
@@ -470,7 +540,7 @@ static int speex_get_samplerate(struct oggdec_vars *self)
 
 /* oggscan_eos: perform a binary search on the ogg file for the e_o_s page
  * and log details of the current logical stream when it is found */
-static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_end, int serial)
+static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_end, int serial, int depth)
    {
    char  *buffer;
    size_t bytes;
@@ -478,16 +548,23 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
    off_t  midpoint = (offset_end - offset) / 2 + offset;
    off_t  stored_mid = midpoint;
    int eos = FALSE, terminate = FALSE;
+   
+   if (++depth >= 40)
+      {
+      fprintf(stderr, "maximum recursion depth %d reached on oggscan_eos\n", depth);
+      return -1;
+      } 
 
    fseeko(self->fp, midpoint, SEEK_SET);
    ogg_sync_reset(&self->oy);
+
    while ((retval = ogg_sync_pageseek(&self->oy, &self->og)) <= 0)
       {
       if (retval < 0)
          {
          midpoint -= retval;
          if (midpoint >= offset_end)
-            return oggscan_eos(self, offset, stored_mid, serial);
+            return oggscan_eos(self, offset, stored_mid, serial, depth);
          }
       else
          {
@@ -496,6 +573,8 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
          ogg_sync_wrote(&self->oy, bytes);
          if (bytes == 0)
             {
+            if (offset_end > midpoint)
+               return oggscan_eos(self, offset, midpoint, serial, depth);
             fprintf(stderr, "oggscan_eos: unexpected file io error, the file is probably truncated\n");
             terminate = TRUE;
             midpoint = offset_end;
@@ -521,6 +600,8 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
          self->artist[self->n_streams - 1] = strdup("");
          self->title  = realloc(self->title,  self->n_streams * sizeof (char *));
          self->title[self->n_streams - 1] = strdup("");
+         self->replaygain = realloc(self->replaygain, self->n_streams * sizeof (char *));
+         self->replaygain[self->n_streams - 1] = strdup("");
          self->streamtype = realloc(self->streamtype, self->n_streams * sizeof (enum streamtype_t));
          self->start_time = realloc(self->start_time, self->n_streams * sizeof (double));
          self->duration = realloc(self->duration, self->n_streams * sizeof (double));
@@ -539,7 +620,7 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
          }
       
       /* seek to the right next time */
-      return oggscan_eos(self, midpoint, offset_end, serial);
+      return oggscan_eos(self, midpoint, offset_end, serial, depth);
       }
    else
       {
@@ -549,7 +630,7 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
          return -1;
          }
       /* seek to the left next time */
-      return oggscan_eos(self, offset, midpoint, serial);
+      return oggscan_eos(self, offset, midpoint, serial, depth);
       }
    }
 
@@ -582,7 +663,7 @@ static off_t oggscan(struct oggdec_vars *self, off_t *offset, off_t offset_end)
       }
 
    serial = ogg_page_serialno(&self->og);
-   return oggscan_eos(self, *offset, offset_end, serial);
+   return oggscan_eos(self, *offset, offset_end, serial, 0);
    }
 
 static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
@@ -834,7 +915,7 @@ void oggdecode_seek_to_packet(struct oggdec_vars *self)
 void oggdecode_dynamic_dispatcher(struct xlplayer *xlplayer)
    {
    struct oggdec_vars *s = xlplayer->dec_data;
-   int success = 0;
+   int success = 0, delay;
    
    while (s->ix < s->n_streams)
       {
@@ -866,15 +947,20 @@ void oggdecode_dynamic_dispatcher(struct xlplayer *xlplayer)
 
       if (success)
          {
+         if (xlplayer->usedelay)
+            delay = xlplayer_calc_rbdelay(xlplayer);
+         else
+            delay = 0;
+         
          if (s->artist[s->ix][0] && s->title[s->ix][0])
-            xlplayer_set_dynamic_metadata(xlplayer, DM_SPLIT_U8, s->artist[s->ix], s->title[s->ix], "", xlplayer->usedelay);
+            xlplayer_set_dynamic_metadata(xlplayer, DM_SPLIT_U8, s->artist[s->ix], s->title[s->ix], "", delay);
          else
             if (s->title[s->ix][0])
-               xlplayer_set_dynamic_metadata(xlplayer, DM_JOINED_U8, "", "", s->title[s->ix], xlplayer->usedelay);
+               xlplayer_set_dynamic_metadata(xlplayer, DM_JOINED_U8, "", "", s->title[s->ix], delay);
             else
                {
                fprintf(stderr, "oggdecode_dynamic_dispatcher: insufficient metadata\n");
-               xlplayer_set_dynamic_metadata(xlplayer, DM_NOTAG, "", "", "", xlplayer->usedelay);
+               xlplayer_set_dynamic_metadata(xlplayer, DM_NOTAG, "", "", "", delay);
                }
          
          xlplayer->usedelay = TRUE;
@@ -963,7 +1049,7 @@ int oggdecode_reg(struct xlplayer *xlplayer)
       }
    }
 
-int oggdecode_get_metainfo(char *pathname, char **artist, char **title, double *length)
+int oggdecode_get_metainfo(char *pathname, char **artist, char **title, double *length, char **replaygain)
    {
    struct oggdec_vars *self;
    int has_pbtime;
@@ -1009,6 +1095,18 @@ int oggdecode_get_metainfo(char *pathname, char **artist, char **title, double *
             *title = realloc(*title, 1);
             *title[0] = '\0';
             }
+
+         if (self->replaygain[0])
+            {
+            if (*replaygain)
+               free(*replaygain);
+            *replaygain = strdup(self->replaygain[0]);
+            }
+         else
+            {
+            *replaygain = realloc(*replaygain, 1);
+            *replaygain[0] = '\0';
+            }
          }
       }
 
