@@ -1,6 +1,6 @@
 /*
 #   recorder.c: the recording part of the streaming module of idjc
-#   Copyright (C) 2007 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2007-2009 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -143,6 +143,56 @@ static int recorder_write_id3_tag(struct recorder *self, FILE *fp)
    return SUCCEEDED;
    }
    
+static int recorder_create_mp3_cuesheet(struct recorder *self)
+   {
+   struct metadata_item *mi;
+   char *cuepathname, *bp;
+   FILE *fp;
+   int i;
+   
+   i = strrchr(self->pathname, '.') - self->pathname + 1;
+   if (!(cuepathname = malloc(i + 4)))
+      {
+      fprintf(stderr, "recorder_write_mp3_cue_sheet: malloc failure\n");
+      return FAILED;
+      }
+   else
+      strcpy(strncpy(cuepathname, self->pathname, i) + i, "cue");
+   
+   if (!(fp = fopen(cuepathname, "wb")))
+      {
+      fprintf(stderr, "recorder_write_mp3_cue_sheet: failed to open cue sheet file for writing\n");
+      free(cuepathname);
+      return FAILED;
+      }
+
+   fprintf(fp, "TITLE \"%s\"\r\n", self->title);
+   fprintf(fp, "PERFORMER \"Recorded with IDJC\"\r\n");
+   fprintf(fp, "FILE \"%s\" MP3\r\n", strrchr(self->pathname, '/') + 1);
+   
+   for (i = 1, mi = self->mi_first; mi; i++, mi = mi->next)
+      {
+      fprintf(fp, "  TRACK %02d AUDIO\r\n", i);
+      if ((bp = strstr(mi->artist_title, " - ")))
+         {
+         fprintf(fp, "    TITLE \"%s\"\r\n", bp + 3);
+         fprintf(fp, "    PERFORMER \"");
+         if (!(fwrite(mi->artist_title, bp - mi->artist_title, 1, fp)))
+            fprintf(stderr, "error writing cuesheet\n");
+         fputc('"', fp);
+         fputc('\r', fp);
+         fputc('\n', fp);
+         }
+      else
+         fprintf(fp, "    TITLE \"%s\"\r\n", mi->artist_title);
+      fprintf(fp, "    INDEX 01 %02d:%02d:00\r\n", mi->time_offset / 60000, mi->time_offset / 1000 % 60);
+      }
+   
+   fclose(fp);
+   free(cuepathname);
+   return SUCCEEDED;
+   }
+      
 static int recorder_write_vbr_tag(struct recorder *self, FILE *fp)
    {
    int mpeg1_f, mono_f;
@@ -166,13 +216,22 @@ static int recorder_write_vbr_tag(struct recorder *self, FILE *fp)
    mono_f = ((self->first_mp3_header[3] & 0xC0) == 0xC0) ? 1 : 0;
    samples_per_frame = mpeg1_f ? 1152 : 576;
    xing_offset = side_info_table[mpeg1_f][mono_f];
-   fwrite(self->first_mp3_header, 4, 1, fp);
+   if (!fwrite(self->first_mp3_header, 4, 1, fp))
+      return FAILED;
    for (i = 0; i < xing_offset; i++)
+      {
       fputc(0x00, fp);
+      if (ferror(fp))
+         return FAILED;
+      }
    if (self->is_vbr)
-      fwrite("Xing\x00\x00\x00\x07", 8, 1, fp);
+      {
+      if (!(fwrite("Xing\x00\x00\x00\x07", 8, 1, fp)))
+         return FAILED;
+      }
    else
-      fwrite("Info\x00\x00\x00\x03", 8, 1, fp);
+      if (!(fwrite("Info\x00\x00\x00\x03", 8, 1, fp)))
+         return FAILED;
    /* the following calculation is fake for files with varying sample rates
     * however the players which use this value will probably only use it 
     * for calclulating the play duration which will yield the intended result */
@@ -204,13 +263,16 @@ static int recorder_write_vbr_tag(struct recorder *self, FILE *fp)
          seg_prop = (look_ms - mi2->start_offset_ms) / (double)(mi2->finish_offset_ms - mi2->start_offset_ms);
          *ptr = (((seg_prop * mi2->size_bytes) + mi2->byte_offset) / self->bytes_written * 255);
          }
-      fwrite(seek_table, 100, 1, fp);
+      if (!(fwrite(seek_table, 100, 1, fp)))
+         return FAILED;
       if (seek_table[99] == 0xFF)
          fputc('\0', fp);
       }
    frame_fill = framelength - ftell(fp) + initial_offset;
    while (frame_fill-- > 0)	/* this frame is allowed to overrun its bounds */
       fputc('\0', fp);		/* and can do so with very low bitrate, high sample rate */
+   if (ferror(fp))
+      return FAILED;
    return SUCCEEDED;
    }
 
@@ -522,10 +584,12 @@ static void *recorder_main(void *args)
                recorder_display_logged_metadata(self->mi_first);
                recorder_display_logged_metadata2(self->mi2_first);
                recorder_apply_mp3_tags(self);
+               recorder_create_mp3_cuesheet(self);
                recorder_free_metadata(self);
                recorder_free_metadata2(self);
-	       }
+    	       }
             free(self->pathname);
+            free(self->title);
             encoder_unregister_client(self->encoder_op);
             memset(self->first_mp3_header, 0x00, 4);
             self->oldbitrate = 0;
@@ -605,12 +669,14 @@ int recorder_start(struct threads_info *ti, struct universal_vars *uv, void *oth
    /* generate a timestamp filename */
    t = time(NULL);
    tm = localtime(&t);
-   strftime(timestamp, TIMESTAMP_SIZ, "[%Y-%m-%d][%H:%M:%S]", tm); 
+   strftime(timestamp, TIMESTAMP_SIZ, "[%Y-%m-%d][%H:%M:%S]", tm);
+   self->title = strdup(timestamp);
    snprintf(self->pathname, pathname_size, "%s/idjc.%s.%02d%s", rv->record_folder, timestamp, uv->tab+1, file_extension);
    if (!(self->fp = fopen(self->pathname, "w")))
       {
       fprintf(stderr, "recorder_start: failed to open file %s\nuser should check file permissions on the particular directory\n", rv->record_folder);
       free(self->pathname);
+      free(self->title);
       encoder_unregister_client(self->encoder_op);
       return FAILED;
       }
