@@ -24,13 +24,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
+#ifdef HAVE_SHOUT_SHOUT_H
+   #include <shout/shout.h>
+#else
+   #include <shout.h>
+#endif /* SHOUT_SHOUT_H */
 #include "sourceclient.h"
 
-#if   defined HAVE_SHOUT_SHOUT_H
-#include <shout/shout.h>
-#elif defined HAVE_SHOUT_H
-#include <shout.h>
-#endif  /*HAVE_SHOUT_SHOUT_H*/
 /* other versions of libshout define SHOUT_FORMAT_VORBIS instead */
 #ifndef SHOUT_FORMAT_OGG
 #define SHOUT_FORMAT_OGG SHOUT_FORMAT_VORBIS
@@ -45,6 +45,7 @@ static void *streamer_main(void *args)
    struct timespec ms10 = { 0, 10000000 };
    struct encoder_op_packet *packet;
    char buffer[10];
+   size_t data_size;
    
    char *s_conv(unsigned long value)
       {
@@ -66,9 +67,8 @@ static void *streamer_main(void *args)
                case SHOUTERR_BUSY:
                   if (!(self->watchdog_info.tick & 0x3F))
                      {
-                     fprintf(stderr, "streamer_main: waiting for the server to grant access\n");
+                     fprintf(stderr, "streamer_main: busy, waiting for the server to grant access\n");
                      self->shout_status = shout_get_connected(self->shout);
-                     fprintf(stderr, "streamer_main: shout status %ld %s\n", self->shout_status, shout_get_error(self->shout));
                      }
                   if (self->disconnect_request)
                      self->stream_mode = SM_DISCONNECTING;
@@ -126,15 +126,21 @@ static void *streamer_main(void *args)
                   if (packet->header.flags & (PF_OGG | PF_MP3))
                      {
                      if ((packet->header.flags & (PF_HEADER | PF_FINAL)) || shout_queuelen(self->shout) < self->max_shout_queue)
-                        {
-                        if (shout_send(self->shout, packet->data, packet->header.data_size) != SHOUTERR_SUCCESS)
-                           {
-                           fprintf(stderr, "streamer_main: failed writing to stream\n");
-                           self->stream_mode = SM_DISCONNECTING;
-                           }
-                        }
+                        data_size = packet->header.data_size;
                      else
+                        {
+                        data_size = 0;
                         fprintf(stderr, "streamer_main: **** packet dumped due to buffer being full ****\n");
+                        }
+                     switch(shout_send(self->shout, packet->data, data_size))
+                        {
+                        case SHOUTERR_SUCCESS:
+                        case SHOUTERR_BUSY:
+                           break;
+                        default:
+                           fprintf(stderr, "streamer_main: failed writing to stream, shout_get_error reports: %s\n", shout_get_error(self->shout));
+                           self->stream_mode = SM_DISCONNECTING;
+                        }
                      }
                   if (packet->header.flags & PF_FINAL)
                      fprintf(stderr, "streamer_main: final packet with serial %d\n", packet->header.serial);
@@ -148,8 +154,15 @@ static void *streamer_main(void *args)
                   {
                   fprintf(stderr, "streamer_main: packet is mp3 metadata: %s\n", (char *)packet->data);
                   shout_metadata_add(self->shout_meta, "song", packet->data);
-                  if (shout_set_metadata(self->shout, self->shout_meta) != SHOUTERR_SUCCESS)
-                     fprintf(stderr, "streamer_main: failed writing metadata to stream\n");
+                  switch (shout_set_metadata(self->shout, self->shout_meta))
+                     {
+                     case SHOUTERR_SUCCESS:
+                     case SHOUTERR_BUSY:
+                        break;
+                     default:
+                        fprintf(stderr, "streamer_main: failed writing metadata to stream, shout_get_error reports: %s\n", shout_get_error(self->shout));
+                        self->stream_mode = SM_DISCONNECTING;
+                     }
                   }
                encoder_client_free_packet(packet);
                }
@@ -312,6 +325,23 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
       sce("genre");
       goto error;
       }
+#ifdef ENH_SHOUT
+   if (shout_set_irc(self->shout, sv->irc) != SHOUTERR_SUCCESS)
+      {
+      sce("irc");
+      goto error;
+      }
+   if (shout_set_aim(self->shout, sv->aim) != SHOUTERR_SUCCESS)
+      {
+      sce("aim");
+      goto error;
+      }
+   if (shout_set_icq(self->shout, sv->icq) != SHOUTERR_SUCCESS)
+      {
+      sce("icq");
+      goto error;
+      }
+#endif /* ENH_SHOUT */
    if (shout_set_public(self->shout, !strcmp(sv->make_public, "True")) != SHOUTERR_SUCCESS)
       {
       sce("make public");
