@@ -35,6 +35,9 @@ struct streamer_vars
    char *listen_url;
    char *description;
    char *genre;
+   char *irc;
+   char *aim;
+   char *icq;
    char *make_public;
    };
 
