@@ -1,6 +1,6 @@
 /*
 #   xlplayer.c: player decoder module for idjc
-#   Copyright (C) 2006-2007 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2006-2009 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -105,10 +105,10 @@ sample_t xlplayer_get_next_gain(struct xlplayer *self)
       {
       val = fade_table[self->fadein_index];
       self->fadein_index -= self->fadeinstep;
-      return val;
+      return val * self->gain;
       }
    else
-      return 1.0f;
+      return self->gain;
    }
 
 /* xlplayer_demux_channel_data: this is where down/upmixing is performed - audio split to 2 channels */
@@ -580,11 +580,11 @@ void xlplayer_destroy(struct xlplayer *self)
       }
    }
 
-int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s, int size)
+int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s, int size, float gain_db)
    {
-   if (self->playmode == PM_PLAYING)
-      xlplayer_eject(self);
+   xlplayer_eject(self);
    self->pathname = pathname;
+   self->gain = pow(10.0, gain_db / 20.0);
    self->seek_s = seek_s;
    self->size = size;
    self->loop = FALSE;
@@ -601,8 +601,7 @@ int xlplayer_playmany(struct xlplayer *self, char *playlist, int loop_f)
    char *start = playlist, *end;
    int payloadlen, i;
 
-   if (self->playmode == PM_PLAYING)
-      xlplayer_eject(self);
+   xlplayer_eject(self);
    /* this is where we parse the playlist starting with getting the number of entries */
    while (*start++ != '#');
    start[-1] = '\0';
@@ -633,6 +632,7 @@ int xlplayer_playmany(struct xlplayer *self, char *playlist, int loop_f)
          }
       start = end;
       }
+   self->gain = 1.0;
    self->seek_s = 0;
    self->loop = loop_f;
    self->playlistmode = TRUE;
@@ -642,12 +642,12 @@ int xlplayer_playmany(struct xlplayer *self, char *playlist, int loop_f)
    return self->initial_audio_context;
    }
 
-int xlplayer_play_noflush(struct xlplayer *self, char *pathname, int seek_s, int size)
+int xlplayer_play_noflush(struct xlplayer *self, char *pathname, int seek_s, int size, float gain_db)
    {
    self->noflush = TRUE;
-   if (self->playmode == PM_PLAYING)
-      xlplayer_eject(self);
+   xlplayer_eject(self);
    self->pathname = pathname;
+   self->gain = pow(10.0, gain_db / 20.0);
    self->seek_s = seek_s;
    self->size = size;
    self->loop = FALSE;
@@ -852,10 +852,14 @@ size_t read_from_player(struct xlplayer *self, sample_t *left_buf, sample_t *rig
    return todo;
    }
 
-void xlplayer_set_dynamic_metadata(struct xlplayer *xlplayer, enum metadata_t type, char *artist, char *title, char *artist_title, int usedelay)
+int xlplayer_calc_rbdelay(struct xlplayer *xlplayer)
+   {
+   return jack_ringbuffer_read_space(xlplayer->left_ch) * 1000 / (sizeof (sample_t) * xlplayer->samplerate);
+   }
+
+void xlplayer_set_dynamic_metadata(struct xlplayer *xlplayer, enum metadata_t type, char *artist, char *title, char *artist_title, int delay)
    {
    struct xlp_dynamic_metadata *dm = &(xlplayer->dynamic_metadata);
-   int min;
    
    pthread_mutex_lock(&(dm->meta_mutex));
    dm->data_type = type;
@@ -869,7 +873,6 @@ void xlplayer_set_dynamic_metadata(struct xlplayer *xlplayer, enum metadata_t ty
    dm->title = strdup(title);
    dm->artist_title = strdup(artist_title);
    dm->current_audio_context = xlplayer->current_audio_context;
-   min = (xlplayer->rbdelay < xlplayer->play_progress_ms) ? xlplayer->rbdelay : xlplayer->play_progress_ms;
-   dm->rbdelay = min * usedelay * 9 / 10;
+   dm->rbdelay = delay;
    pthread_mutex_unlock(&(dm->meta_mutex));
    }
