@@ -1,6 +1,6 @@
 /*
 #   xlplayer.h: player decoder module for idjc
-#   Copyright (C) 2006-2007 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2006-2009 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -71,6 +71,7 @@ struct xlplayer
    size_t op_buffersize;		/* the current size of the player output buffers */
    char *pathname;			/* the pathname of the music file being played */
    char **playlist;			/* the playlist as an array of pointers */
+   float gain;              /* amount of gain to apply to the playback */
    int loop;				/* flag indicating if we loop or come to a stop */
    int seek_s;				/* the initial seek time of the song in seconds */
    int size;				/* size of the file in seconds */
@@ -140,7 +141,7 @@ void xlplayer_destroy(struct xlplayer *);
 /* xlplayer_play: starts the player on a particular track immediately
 * if a track is currently playing eject is called
 * return value: a context-id for this track */
-int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s, int size);
+int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s, int size, float gain_db);
 
 /* xlplayer_playmany: starts the player on a playlist
 * if a track is currently playing eject is called, also can set looping with this function
@@ -148,7 +149,7 @@ int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s, int size);
 int xlplayer_playmany(struct xlplayer *self, char *playlist, int loop_f);
 
 /* xlplayer_play_noflush: starts the player without flushing out old data from the ringbuffer */
-int xlplayer_play_noflush(struct xlplayer *self, char *pathname, int seek_s, int size);
+int xlplayer_play_noflush(struct xlplayer *self, char *pathname, int seek_s, int size, float gain_db);
 
 /* xlplayer_cancelplaynext: cancels the automatic playing of the next track 
 * the current track is allowed to continue playing */
@@ -186,7 +187,10 @@ void xlplayer_demux_channel_data(struct xlplayer *self, jack_default_audio_sampl
 void xlplayer_write_channel_data(struct xlplayer *self);
 
 /* provide data for sending back to the user interface */
-void xlplayer_set_dynamic_metadata(struct xlplayer *xlplayer, enum metadata_t type, char *artist, char *title, char *artist_title, int usedelay);
+void xlplayer_set_dynamic_metadata(struct xlplayer *xlplayer, enum metadata_t type, char *artist, char *title, char *artist_title, int delay);
+
+/* return the delay caused by the ringbuffer */
+int xlplayer_calc_rbdelay(struct xlplayer *xlplayer);
 
 /* this sets the speed of fading for a particular mode */
 void xlplayer_set_fadesteps(struct xlplayer *self, int fade_step);
