@@ -1,4 +1,4 @@
-#   IDJCfreefuncs.py: Free functions used by IDJC
+#   IDJCfree.py: Free functions used by IDJC
 #   Copyright (C) 2005-2007 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -15,6 +15,10 @@
 #   along with this program in the file entitled COPYING.
 #   If not, see <http://www.gnu.org/licenses/>.
 
+import pygtk
+pygtk.require("2.0")
+import gtk
+
 import os, encodings.hex_codec
 
 # Convert characters that have special meaning in pango markup language to their safe equivalents
@@ -84,3 +88,11 @@ class int_object:		# Putting an int in a class allows its use in a dictionary
       return self.value
    def set_text(self, value):
       self.value = value
+
+def threadslock(f):
+   def newf(*args, **kwargs):
+      gtk.gdk.threads_enter()
+      r = f(*args, **kwargs)
+      gtk.gdk.threads_leave()
+      return r
+   return newf
