@@ -173,8 +173,10 @@ class jingles:
          self.entry.set_sensitive(True)
          print "Stop player"
          if flush == True:
+            print "stop with flush"
             self.parent.mixer_write("ACTN=stopjingles\nend\n", True)
          else:
+            print "stop without flush"
             self.stop.clicked()	# this will take care of resetting the play button without triggering a flush
          if self.nomute == False:
             self.parent.deckadj.set_value(self.volume)
@@ -290,6 +292,19 @@ class jingles:
       self.jingleswinx.set_value(event.width)
       self.jingleswiny.set_value(event.height)
       
+   def trigger_index(self, index):
+      if index == -1:
+         if self.playing:
+            self.stop.clicked()
+            self.entry.set_text("1")
+            self.play.set_active(True)
+            self.stop.clicked()
+         return
+      if index < len(self.liststore):
+         self.trigger_index(-1)
+         self.entry.set_text(str(index + 1))
+         self.play.set_active(True)
+
    def __init__(self, parent):
       self.parent = parent
       self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
@@ -299,6 +314,8 @@ class jingles:
       self.window.set_border_width(8)
       self.window.connect("delete_event", self.delete_event)
       self.window.set_icon_from_file(pkgdatadir + "icon" + gfext)
+      self.window.add_events(gtk.gdk.KEY_PRESS_MASK)
+      self.window.connect("key-press-event", parent.cb_key_capture)
 
       hbox = gtk.HBox()
       hbox.set_spacing(8)
