@@ -787,7 +787,18 @@ class mixprefs:
                   self.r_unp_gain.set_sensitive(False)
                if self.r_unp_active.get_active():
                   self.l_unp_gain.set_sensitive(False)
-   
+
+   def cb_rg_indicate(self, widget):
+      left = self.parent.player_left
+      right = self.parent.player_right
+      
+      if widget.get_active():
+         left.treeview.insert_column(left.rgtvcolumn, 0)
+         right.treeview.insert_column(right.rgtvcolumn, 0)
+      else:         
+         left.treeview.remove_column(left.rgtvcolumn)
+         right.treeview.remove_column(right.rgtvcolumn)
+         
    def __init__(self, parent):
       self.parent = parent
       self.parent.prefs_window = self
@@ -903,6 +914,61 @@ class mixprefs:
       outervbox.pack_start(frame, False, False, 0)
       frame.show()
       
+      # Replay Gain controls
+      
+      frame = gtk.Frame(" " + ln.rg_title + " ")
+      frame.set_border_width(3)
+      outervbox.pack_start(frame, False, False, 0)
+      vbox = gtk.VBox()
+      frame.add(vbox)
+      frame.show()
+      vbox.set_border_width(10)
+      vbox.set_spacing(1)
+      vbox.show()
+      
+      self.rg_indicate = gtk.CheckButton(ln.rg_indicate)
+      parent.tooltips.set_tip(self.rg_indicate, ln.rg_indicate_tip)
+      self.rg_indicate.connect("toggled", self.cb_rg_indicate)
+      vbox.pack_start(self.rg_indicate, False, False, 0)
+      self.rg_indicate.show()
+      
+      self.rg_adjust = gtk.CheckButton(ln.rg_adjust)
+      parent.tooltips.set_tip(self.rg_adjust, ln.rg_adjust_tip)
+      vbox.pack_start(self.rg_adjust, False, False, 0)
+      self.rg_adjust.show()
+      
+      hbox = gtk.HBox()
+      hbox.set_spacing(3)
+      spacer = gtk.HBox()
+      hbox.pack_start(spacer, False, False, 16)
+      spacer.show()
+      label = gtk.Label(ln.rg_defaultgain)
+      hbox.pack_start(label, False, False, 0)
+      label.show()
+      rg_defaultgainadj = gtk.Adjustment(-8.0, -20.0, 10.0, 0.1)
+      self.rg_defaultgain = gtk.SpinButton(rg_defaultgainadj, 0.0, 1)
+      parent.tooltips.set_tip(hbox, ln.rg_defaultgain_tip)
+      hbox.pack_start(self.rg_defaultgain, False, False, 0)
+      self.rg_defaultgain.show()
+      vbox.pack_start(hbox, False, False, 0)
+      hbox.show()
+
+      hbox = gtk.HBox()
+      hbox.set_spacing(3)
+      spacer = gtk.HBox()
+      hbox.pack_start(spacer, False, False, 16)
+      spacer.show()
+      label = gtk.Label(ln.rg_boost)
+      hbox.pack_start(label, False, False, 0)
+      label.show()
+      rg_boostadj = gtk.Adjustment(6.0, -5.0, 15.5, 0.5)
+      self.rg_boost = gtk.SpinButton(rg_boostadj, 0.0, 1)
+      parent.tooltips.set_tip(hbox, ln.rg_boost_tip)
+      hbox.pack_start(self.rg_boost, False, False, 0)
+      self.rg_boost.show()
+      vbox.pack_start(hbox, False, False, 0)
+      hbox.show()
+
       # Miscellaneous Features
       
       frame = gtk.Frame(" " + ln.misc_features + " ")
@@ -1692,14 +1758,9 @@ class mixprefs:
        
       # Jack settings Tab      
                  
-      #jack_window = gtk.ScrolledWindow()
-      #jack_window.set_border_width(8)
-      #jack_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
       jack_vbox = gtk.VBox()
       jack_vbox.set_spacing(3)
       jack_vbox.set_border_width(4)
-      #jack_window.add_with_viewport(jack_vbox)
-      #jack_window.show()
       
       jackname = os.environ["IDJC_JACK_SERVER"]
       if jackname != "default":
@@ -1903,7 +1964,7 @@ class mixprefs:
       
       self.notebook.show()
 
-      # These shown by default
+      # These on by default
       self.mic_peak_toggle.set_active(True)
       self.stream_peak_toggle.set_active(True)
       self.limiter_toggle.set_active(True)
@@ -1918,78 +1979,80 @@ class mixprefs:
       
       self.playersettingsdict = {	# Settings of these will be saved in the config file 
          "lplayall"      : self.lplayall,	# These are all True/False values
-	 "lloopall"      : self.lloopall,
-	 "lrandom"       : self.lrandom,
-	 "lmanual"       : self.lmanual,
-         "lcueup"	 : self.lcueup,
-	 "lcountup"      : self.lcountup,
-	 "lcountdown"    : self.lcountdown,
-	 "lstream"       : self.lstream,
-	 "llisten"       : self.llisten,
-	 "rplayall"      : self.rplayall,
-	 "rloopall"      : self.rloopall,
-	 "rrandom"       : self.rrandom,
-	 "rmanual"       : self.rmanual,
-         "rcueup"	 : self.rcueup,
-	 "rcountdown"    : self.rcountdown,
-	 "rstream"       : self.rstream,
-	 "rlisten"       : self.rlisten,
-	 "startmini"     : self.startmini,
-	 "micpeak"	 : self.mic_peak_toggle,
-	 "vumeters"	 : self.vu_toggle,
-	 "streampeak"	 : self.stream_peak_toggle,
-	 "limitergain"   : self.limiter_toggle,
+         "lloopall"      : self.lloopall,
+         "lrandom"       : self.lrandom,
+         "lmanual"       : self.lmanual,
+         "lcueup"        : self.lcueup,
+         "lcountup"      : self.lcountup,
+         "lcountdown"    : self.lcountdown,
+         "lstream"       : self.lstream,
+         "llisten"       : self.llisten,
+         "rplayall"      : self.rplayall,
+         "rloopall"      : self.rloopall,
+         "rrandom"       : self.rrandom,
+         "rmanual"       : self.rmanual,
+         "rcueup"        : self.rcueup,
+         "rcountdown"    : self.rcountdown,
+         "rstream"       : self.rstream,
+         "rlisten"       : self.rlisten,
+         "startmini"     : self.startmini,
+         "micpeak"       : self.mic_peak_toggle,
+         "vumeters"      : self.vu_toggle,
+         "streampeak"	 : self.stream_peak_toggle,
+         "limitergain"   : self.limiter_toggle,
          "strhealth"     : self.stream_status_toggle,
-	 "announce_en"	 : self.announce_enable,
-	 "timer_en"	 : self.timer_enable,
-         "djalarm"	 : self.djalarm,
-         "trxpld"	 : self.tracks_played,
-         "strmon"	 : self.stream_mon,
+         "announce_en"	 : self.announce_enable,
+         "timer_en"      : self.timer_enable,
+         "djalarm"       : self.djalarm,
+         "trxpld"        : self.tracks_played,
+         "strmon"        : self.stream_mon,
          "bigdigibox"	 : self.bigger_box_toggle, 
          "normalize"	 : self.normalize,
-         "dither"	 : self.dither,
+         "dither"        : self.dither,
          "recallsession" : self.restore_session_option,
          "proktoggle"    : self.p3prefs.proktoggle,
          "ee_appstart"	 : self.appstart_event,
          "ee_appexit"    : self.appexit_event,
-         "ee_micon"	 : self.mic_on_event,
+         "ee_micon"      : self.mic_on_event,
          "ee_micoff"	 : self.mic_off_event,
-         "ee_auxon"	 : self.aux_on_event,
+         "ee_auxon"      : self.aux_on_event,
          "ee_auxoff"	 : self.aux_off_event,
-         "best_rs"	 : self.best_quality_resample,
-         "good_rs"	 : self.good_quality_resample,
-         "fast_rs"	 : self.fast_resample,
+         "best_rs"       : self.best_quality_resample,
+         "good_rs"       : self.good_quality_resample,
+         "fast_rs"       : self.fast_resample,
          "fastest_rs"	 : self.fastest_resample,
          "micauxmutex"	 : self.mic_aux_mutex,
          "speed_var"	 : self.speed_variance,
          "twodblimit"    : self.twodblimit,
          "showtips"      : self.enable_tooltips,
-         "mp3utf8"	 : self.mp3_utf8,
+         "mp3utf8"	     : self.mp3_utf8,
          "mictodj"       : self.mic_to_dj,
          "invertleft"    : self.invert_left,
          "invertright"   : self.invert_right, 
          "lunpactive"    : self.l_unp_active,
          "runpactive"    : self.r_unp_active,
-	 "silencekiller" : self.silence_killer,
+         "silencekiller" : self.silence_killer,
          "bonuskiller"   : self.bonus_killer,
          "unlimretries"  : self.recon_config.unlimited_retries,
          "recondialog"   : self.recon_config.visible,
          "sbfullrecon"   : self.recon_config.attempt_reconnection,
+         "rg_indicate"   : self.rg_indicate,
+         "rg_adjust"     : self.rg_adjust,
          }
          
       self.playersettingsdict.update(self.l_agc_control.booleandict)
       self.playersettingsdict.update(self.r_agc_control.booleandict)
 	 
       self.valuesdict = {
-	 "leftpanadj"    : self.left_pan_adj,
+         "leftpanadj"    : self.left_pan_adj,
          "rightpanadj"   : self.right_pan_adj,
-	 "interval"	 : self.intervaladj,
-         "latency"	 : self.announcedelayadj,
+         "interval"	     : self.intervaladj,
+         "latency"	     : self.announcedelayadj,
          "interval_vol"	 : self.parent.jingles.interadj,
-	 "fullwinx"	 : self.parent.fullwinx,
-         "fullwiny"	 : self.parent.fullwiny,
-         "minwinx"	 : self.parent.minwinx,
-         "minwiny"	 : self.parent.minwiny,
+         "fullwinx"      : self.parent.fullwinx,
+         "fullwiny"      : self.parent.fullwiny,
+         "minwinx"       : self.parent.minwinx,
+         "minwiny"       : self.parent.minwiny,
          "jingleswinx"   : self.parent.jingles.jingleswinx,
          "jingleswiny"   : self.parent.jingles.jingleswiny,
          "passspeed"	 : self.parent.passspeed_adj,
@@ -1997,10 +2060,12 @@ class mixprefs:
          "normceiling"	 : self.normceiling_adj,
          "normrisetc"	 : self.normrise_adj,
          "normfalltc"	 : self.normfall_adj, 
-         "djvolume"	 : self.dj_aud_adj,
+         "djvolume"      : self.dj_aud_adj,
          "headroom"      : self.headroom,
          "lunpgain"      : self.l_unp_gain,
-         "runpgain"      : self.r_unp_gain
+         "runpgain"      : self.r_unp_gain,
+         "rg_default"    : self.rg_defaultgain,
+         "rg_boost"      : self.rg_boost,
          }
 	 
       self.valuesdict.update(self.l_agc_control.valuesdict) 
@@ -2011,20 +2076,21 @@ class mixprefs:
          "prokdatabase"  : self.p3prefs.prokdatabase,
          "prokpassword"  : self.p3prefs.prokpassword,
          "announcenick"	 : self.nickentry,
-	 "announcechan"	 : self.channelsentry, 
-	 "announcemess"	 : self.announcemessageentry,
-	 "timernick"	 : self.timernickentry,
-	 "timerchan"	 : self.timerchannelsentry,
-	 "timermess"	 : self.timermessageentry,
+         "announcechan"	 : self.channelsentry, 
+         "announcemess"	 : self.announcemessageentry,
+         "timernick"	 : self.timernickentry,
+         "timerchan"	 : self.timerchannelsentry,
+         "timermess"	 : self.timermessageentry,
          "ltfilerqdir"   : self.parent.player_left.file_requester_start_dir,
          "rtfilerqdir"   : self.parent.player_right.file_requester_start_dir,
          "et_appstart"	 : self.appstart_event,
          "et_appexit"    : self.appexit_event,
-         "et_micon"	 : self.mic_on_event,
+         "et_micon"      : self.mic_on_event,
          "et_micoff"	 : self.mic_off_event,
-         "et_auxon"	 : self.aux_on_event,
+         "et_auxon"      : self.aux_on_event,
          "et_auxoff"	 : self.aux_off_event,
          "con_delays"    : self.recon_config.csl,
          }
 
       self.rangewidgets = (self.parent.deckadj,)
+
