@@ -27,8 +27,6 @@ add_file = "Datei hinzufügen"
 
 add_to_jingles = "Zu den Jingles hinzufügen"
 
-af_h_sm = "Audio Filter, Headroom, und Stereo Mix"
-
 all = "Alle"
 
 append = "Unten anhängen"
@@ -127,8 +125,6 @@ item_menu = "Objekt"
 
 jack_ports_tab = "Jack-Anschlüsse"
 
-keep_password_tip = "Choosing this option will cause the server passwords to be saved to the IDJC configuration file so they will be there when you next restart IDJC. If this is a security concern to you it would be wise to turn this feature off. Those doing so will have to type in their server passwords each time IDJC is run."
-
 latency = "Verzögerung:"
 
 left = "Links:"
@@ -495,7 +491,7 @@ use_jack_srate_tip = "No additional resampling will occur. The stream sample rat
 
 #dither_tip = "This feature possibly improves the sound quality a little when listening on a 24 bit sound card."
 
-#dither_toggle = "Apply triangular shaped dither to FLAC playback"
+#dither_toggle = "Apply dither to MP3 and FLAC playback"
 
 #dj_alarm_tip = "An alarm tone alerting the DJ that dead-air is just nine seconds away."
 
