@@ -324,9 +324,9 @@ disconnected_history = "disconnected from server"
 
 disconnection = "IDJC Disconnection"
 
-dither_tip = "This feature possibly improves the sound quality a little when listening on a 24 bit sound card."
+dither_tip = "This feature improves the sound quality a little when listening on a 24 bit sound card."
 
-dither_toggle = "Apply triangular shaped dither to FLAC playback"
+dither_toggle = "Apply dither to MP3 and FLAC playback."
 
 dj_alarm_tip = "An alarm tone alerting the DJ that dead-air is just nine seconds away."
 
@@ -828,6 +828,8 @@ normrise = "Rise"
 
 not_connected = "Not Connected"
 
+notag = "No Tag"
+
 off = "Off"
 
 ogg_streamtab_tip = "Clicking this tab selects the Ogg family of file formats."
@@ -1046,6 +1048,24 @@ restore_session = "Restore previous session"
 
 restore_session_tip = "When starting IDJC most of the main window settings will be as they were left. As an alternative you may specify below how you want the various settings to be when IDJC starts."
 
+rg_indicate = "Indicate which tracks have Replay Gain values."
+
+rg_adjust = "Adjust playback volume."
+
+rg_defaultgain = "Gain value for unmarked tracks:"
+
+rg_defaultgain_tip = "Set this to the typical track gain values you would expect for the programme material you are currently playing. For pop and rock music (especially modern studio recordings) this should be about a -8 or -9 and classical music much closer to zero."
+
+rg_title = "Replay Gain"
+
+rg_boost = "Boost (for pop/rock):"
+
+rg_boost_tip = "For material that is generally loud it is recommended to set this between 4 and 8 dB however going too high will result in a loss of dynamic range. The Str Peak meter is a useful guide for getting this right."
+
+rg_adjust_tip = "Effective only on newly started tracks."
+
+rg_indicate_tip = "Show a marker in the playlists next to each track."
+
 right = "Right:"
 
 right_mic_stereo_tip = "This controls the stereo balance of the right microphone."
