@@ -25,7 +25,7 @@ import gtk
 from ln_text import ln
 from idjc_config import *
 
-class LicenceWindow(gtk.Window):
+class LicenceWindow(gtk.ScrolledWindow):
    gpl_v3_text = """
                     GNU GENERAL PUBLIC LICENSE
                        Version 3, 29 June 2007
@@ -706,21 +706,19 @@ Public License instead of this License.  But first, please read
       self.hide()
       return True
    def __init__(self):
-      gtk.Window.__init__(self)
-      self.set_title(ln.licence_window_title)
-      self.set_icon_from_file(pkgdatadir + "icon" + gfext)
-      self.set_size_request(620, 800)
-      self.connect("delete-event", self.cb_delete_window_event)
-      scrolled_window = gtk.ScrolledWindow()
-      scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
-      self.add(scrolled_window)
+      gtk.ScrolledWindow.__init__(self)
+      self.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
+      self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
       text_view = gtk.TextView()
+      text_view.set_wrap_mode(gtk.WRAP_WORD)
       text_view.set_editable(False)
-      text_view.set_left_margin(50)
-      text_view.set_right_margin(10)
-      scrolled_window.add(text_view)
-      scrolled_window.show()
+      text_view.unset_flags(gtk.CAN_FOCUS)
+      text_view.set_left_margin(8)
+      text_view.set_right_margin(8)
+      self.add(text_view)
       text_buffer = gtk.TextBuffer()
-      text_buffer.set_text(self.gpl_v3_text)
+      tag = text_buffer.create_tag()
+      tag.set_property("font", "sans 8")
+      text_buffer.insert_with_tags(text_buffer.get_start_iter(), self.gpl_v3_text, tag)
       text_view.set_buffer(text_buffer)
       text_view.show()
