@@ -36,7 +36,11 @@ import mutagen.id3 as id3
 from idjc_config import *
 from IDJCfree import *
 from ln_text import ln
-
+from mutagen.mp3 import MP3
+from mutagen.apev2 import APEv2, APETextValue
+from mutagen.musepack import Musepack
+from mutagen.monkeysaudio import MonkeysAudio
+from mutagen.asf import ASF, ASFUnicodeAttribute
 
 def set_tip(*args):
    """Dummy tooltips setter."""
@@ -49,7 +53,8 @@ class LeftLabel(gtk.HBox):
    
    def __init__(self, text):
       gtk.HBox.__init__(self)
-      self.pack_start(gtk.Label(text), False, False, 0)
+      self.label = gtk.Label(text)
+      self.pack_start(self.label, False, False, 0)
 
 
 class RightLabel(gtk.HBox):
@@ -71,7 +76,7 @@ class FreeTagFrame(gtk.Frame):
       self.tb = gtk.TextBuffer()
       tv = gtk.TextView(self.tb)
       tv.set_wrap_mode(gtk.WRAP_CHAR)
-      tv.modify_font(pango.FontDescription('sans 14'))
+      tv.modify_font(pango.FontDescription('sans 12'))
       sw.add(tv)
       tv.show()
 
@@ -83,6 +88,119 @@ class MutagenTagger(gtk.VBox):
       gtk.VBox.__init__(self)
       self.pathname = pathname
 
+class WMATagger(MutagenTagger):
+   """Handles tagging of WMA files"""
+   
+   primary_data = ("Title", "Author")
+   secondaries = ("WM/AlbumTitle", "WM/AlbumArtist", "WM/Year", "WM/Genre")
+   
+   def save_tag(self):
+      """Updates the tag with the GUI data."""
+       
+      tag = self.tag
+      tb = self.tag_frame.tb
+       
+      for key in self.text_set:
+         try:
+            del tag[key]
+         except KeyError:
+            pass
+
+      for each in self.primary_line:
+         val = each[1].get_text().strip()
+         if val:
+            tag[each[0]] = val
+         else:
+            try:
+               del tag[each[0]]
+            except KeyError:
+               pass
+             
+      lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).splitlines()
+      for line in lines:
+         try:
+            key, val = line.split("=", 1)
+         except ValueError:
+            continue
+         else:
+            key = key.strip()
+            val = val.strip()
+            if val:
+               try:
+                  tag[key] += [ASFUnicodeAttribute(val.decode("utf-8"))]
+               except (KeyError, AttributeError):
+                  try:
+                     tag[key] = [ASFUnicodeAttribute(val.decode("utf-8"))]
+                  except KeyError:
+                     print "Unacceptable key", key
+      tag.save()
+   
+   def load_tag(self):
+      """(re)Writes the tag data to the GUI."""
+      
+      tag = self.tag
+      
+      for each in self.primary_line:
+         try:
+            data = tag[each[0]]
+         except KeyError:
+            pass
+         else:
+            each[1].set_text("/".join(unicode(y) for y in data))
+
+      additional = []
+
+      for key in self.secondaries:
+         values = tag.get(key, [ASFUnicodeAttribute("")])
+         for val in values:
+            additional.append(key.encode("utf-8") + "=" + unicode(val).encode("utf-8"))
+
+      for key in self.text_set:
+         if key not in self.primary_data and key not in self.secondaries:
+            values = tag[key]
+            for val in values:
+               additional.append(key.encode("utf-8") + "=" + unicode(val).encode("utf-8"))
+      
+      self.tag_frame.tb.set_text("\n".join(additional))
+   
+   def __init__(self, pathname):
+      MutagenTagger.__init__(self, pathname)
+      try:
+         self.tag = mutagen.asf.ASF(pathname)
+         if not isinstance(self.tag, mutagen.asf.ASF):
+            raise mutagen.asf.error
+      except mutagen.asf.error:
+         print "Not a real wma/asf file apparently."
+         self.tag = None
+         return
+         
+      hbox = gtk.HBox()
+      hbox.set_border_width(5)
+      hbox.set_spacing(8)
+      self.pack_start(hbox, False, False, 0)
+      vbox_text = gtk.VBox()
+      hbox.pack_start(vbox_text, False, False, 0)
+      vbox_entry = gtk.VBox()
+      hbox.pack_start(vbox_entry, True, True, 0)
+      
+      self.primary_line = []
+      for text, entry in ((x, gtk.Entry()) for x in self.primary_data):
+         self.primary_line.append((text, entry))
+         vbox_text.add(LeftLabel(text))
+         vbox_entry.add(entry)
+      hbox.show_all()
+
+      self.tag_frame = FreeTagFrame()
+      self.tag_frame.set_border_width(5)
+      self.add(self.tag_frame)
+      self.tag_frame.show()
+
+      self.text_set = []
+
+      for key, val in self.tag.iteritems():
+         if key not in self.primary_line and all(isinstance(v, (ASFUnicodeAttribute, unicode)) for v in val):
+            self.text_set.append(key)
+
 
 class ID3Tagger(MutagenTagger):
    """ID3 tagging with Mutagen."""
@@ -179,13 +297,13 @@ class ID3Tagger(MutagenTagger):
                additional.append(fid + sep + text.encode("utf-8"))
             
       self.tag_frame.tb.set_text("\n".join(additional))
-          
+      
    def __init__(self, pathname, force=False):
       MutagenTagger.__init__(self, pathname)
       if force:
          try:
             self.tag = mutagen.File(pathname)
-            if not isinstance(self.tag, mutagen.mp3.MP3):
+            if not isinstance(self.tag, MP3):
                raise mutagen.mp3.error
          except mutagen.mp3.error:
             print "Not a real mp3 file apparently."
@@ -232,7 +350,7 @@ class ID3Tagger(MutagenTagger):
 
 
 class MP4Tagger(MutagenTagger):
-   """ID3 tagging with Mutagen."""
+   """MP4 tagging with Mutagen."""
    
    primary_data = (("\xa9nam", ln.mp4title), ("\xa9ART", ln.mp4artist),
                    ("\xa9alb", ln.mp4album), ("trkn", ln.mp4track),
@@ -320,7 +438,7 @@ class MP4Tagger(MutagenTagger):
 
 
 class NativeTagger(MutagenTagger):
-   """Native format tagging with Mutagen."""
+   """Native format tagging with Mutagen. Mostly FLAC and Ogg."""
    
    blacklist = "coverart", "metadata_block_picture"
    
@@ -382,8 +500,8 @@ class NativeTagger(MutagenTagger):
    def __init__(self, pathname, ext):
       MutagenTagger.__init__(self, pathname)
       self.tag = mutagen.File(pathname)
-      if isinstance(self.tag, mutagen.mp3.MP3):
-         # MP3s handled with the ID3 tagger, not this.
+      if isinstance(self.tag, (MP3, APEv2)):
+         # MP3 and APEv2 have their own specialised tagger.
          self.tag = None
          return
       
@@ -392,10 +510,101 @@ class NativeTagger(MutagenTagger):
       self.tag_frame.show()
 
 
+class ApeTagger(MutagenTagger):
+   """APEv2 tagging with Mutagen."""
+   
+   opener = {"ape": MonkeysAudio, "mpc": Musepack }
+   
+   def save_tag(self):
+      """Updates the tag with the GUI data."""
+      
+      tag = self.tag
+      
+      for key, values in tag.iteritems():
+         if isinstance(values, APETextValue):
+            del tag[key]
+            
+      tb = self.tag_frame.tb
+      lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).splitlines()
+      
+      for line in lines:
+         try:
+            key, val = line.split("=", 1)
+         except ValueError:
+            continue
+         else:
+            key = key.strip()
+            val = val.strip()
+            if val:
+               try:
+                  tag[key].value += "\0" + val.decode("utf-8")
+               except (KeyError, AttributeError):
+                  try:
+                     tag[key] = APETextValue(val.decode("utf-8"), 0)
+                  except KeyError:
+                     print "Unacceptable key", key
+   
+      tag.save() 
+   
+   def load_tag(self):
+      """(re)Writes the tag data to the GUI."""
+      
+      tag = self.tag
+      lines = []
+      primaries = "TITLE", "ARTIST", "AUTHOR", "ALBUM",\
+                          "TRACKNUMBER", "TRACKTOTAL", "GENRE", "DATE"
+      
+      for key in primaries:
+         try:
+            values = tag[key]
+         except KeyError:
+            lines.append(key + "=")
+         else:
+            for val in values:
+               lines.append(key + "=" + val.encode("utf-8"))
+
+      for key, values in tag.iteritems():
+         if key not in primaries and isinstance(values, APETextValue):
+            for val in values:
+               lines.append(key + "=" + val.encode("utf-8"))
+            
+      self.tag_frame.tb.set_text("\n".join(lines))
+   
+   def __init__(self, pathname, extension):
+      MutagenTagger.__init__(self, pathname)
+      
+      try:
+         self.tag = self.opener[extension](pathname)
+      except KeyError:
+         try:
+            self.tag = APEv2(pathname)
+         except:
+            print "ape tag not found"
+            self.tag = None
+            return
+         else:
+            print "ape tag found on non-native format"
+      except:
+         print "failed to create tagger for native format"
+         self.tag = None
+         return
+      else:
+         try:
+            self.tag.add_tags()
+         except:
+            print "ape tag found on native format"
+         else:
+            print "no existing ape tags found"
+         
+      self.tag_frame = FreeTagFrame()
+      self.add(self.tag_frame)
+      self.tag_frame.show()
+
+
 class MutagenGUI:
    ext2name = { "mp3": "ID3", "mp4": "MP4", "m4a": "MP4", "spx": "Speex",
                "flac": "FLAC", "ogg": "Ogg Vorbis", "oga": "XIPH Ogg audio",
-               "m4b": "MP4", "m4p": "MP4" }
+               "m4b": "MP4", "m4p": "MP4", "wma": "Windows Media Audio" }
    
    def destroy_and_quit(self, widget, data = None):
       gtk.main_quit()
@@ -410,8 +619,7 @@ class MutagenGUI:
    def is_supported(pathname):
       supported = [ "mp3", "ogg", "oga" ]
       if avcodec and avformat:
-         supported.append("mp4")
-         supported.append("m4a")
+         supported += ["mp4", "m4a", "m4b", "m4p", "ape", "mpc", "wma"]
       if flacenabled:
          supported.append("flac")
       if speexenabled:
@@ -485,6 +693,8 @@ class MutagenGUI:
       vbox.pack_start(notebook, True, True, 0)
       notebook.show()
       
+      self.ape = ApeTagger(pathname, extension)
+      
       if extension == "mp3":
          self.id3 = ID3Tagger(pathname, True)
          self.native = None
@@ -492,6 +702,11 @@ class MutagenGUI:
          self.id3 = ID3Tagger(pathname, False)
          if extension in ("mp4", "m4a", "m4b", "m4p"):
             self.native = MP4Tagger(pathname)
+         elif extension == "wma":
+            self.native = WMATagger(pathname)
+         elif extension in ("ape", "mpc"):
+            # APE tags are native to this format.
+            self.native = None
          else:
             self.native = NativeTagger(pathname, ext=extension)
       
@@ -502,6 +717,13 @@ class MutagenGUI:
          notebook.append_page(self.id3, label)
          self.id3.show()
       
+      if self.ape is not None and self.ape.tag is not None:
+         reload_button.connect("clicked", lambda x: self.ape.load_tag())
+         apply_button.connect("clicked", lambda x: self.ape.save_tag())
+         label = gtk.Label("APE v2")
+         notebook.append_page(self.ape, label)
+         self.ape.show()   
+      
       if self.native is not None and self.native.tag is not None:
          reload_button.connect("clicked", lambda x: self.native.load_tag())
          apply_button.connect("clicked", lambda x: self.native.save_tag())
