@@ -19,8 +19,9 @@ __all__ = ['MediaPane', 'Prefs']
 
 import pygtk
 pygtk.require('2.0')
-import gtk
+import gtk, gobject, time
 from idjc_config import *
+from IDJCfree import threadslock
 from ln_text import ln
 from urllib import quote
 
@@ -34,7 +35,9 @@ def makeview(notebook, label_text, additional = None):
    vbox = gtk.VBox()
    vbox.set_spacing(2)
    scrollwindow = gtk.ScrolledWindow()
+   alternate = gtk.VBox()
    vbox.pack_start(scrollwindow, True, True, 0)
+   vbox.pack_start(alternate, True, True, 0)
    if additional is not None:
       vbox.pack_start(additional, False, False, 0)
    vbox.show()
@@ -46,7 +49,7 @@ def makeview(notebook, label_text, additional = None):
    treeview = gtk.TreeView()
    scrollwindow.add(treeview)
    treeview.show()
-   return treeview
+   return treeview, scrollwindow, alternate
 
 def makecolumns(view, name_ix_rf_mw):
    l = []
@@ -74,6 +77,60 @@ class DNDAccumulator(list):
    def __init__(self):
       list.__init__(self)
 
+class TreePopulate(object):
+   """ runs as an idle process building the tree view of the p3 database """
+   @threadslock
+   def run(self):
+      i = 10
+      append = self.mp.treestore.append
+      d = self.d
+      while i > 0:
+         i -= 1
+         row = self.c.fetchone()
+         if row is None:
+            self.mp.treeview.set_model(self.mp.treestore)
+            self.mp.treealt.hide()
+            self.mp.treescroll.show()
+            self.mp.tree_idle = None
+            retval = False
+            #print "Database parsed in %f seconds" % (time.time() - self.start_time)
+            break
+         else:
+            while 1:
+               if d == 0:
+                  self.art = row[1]
+                  self.artlower = self.art.lower()
+                  self.iter1 = append(None, (-1, self.art, 0, 0, 0, "", "")) 
+                  d = 1
+               if d == 1:
+                  if self.artlower == row[1].lower():
+                     self.alb = row[2]
+                     self.alblower = self.alb.lower()
+                     self.iter2 = append(self.iter1, (-2, self.alb, 0, 0, 0, "", "")) 
+                     d = 2
+                  else:
+                     d = 0
+               if d == 2:
+                  if self.artlower == row[1].lower() and self.alblower == row[2].lower():
+                     append(self.iter2, (row[0], row[4], row[3], row[5], row[6], row[7], row[8]))
+                     break
+                  else:
+                     d = 1
+            retval = True
+      self.d = d
+      self.done += 10.0
+      if int(self.done) % 100 == 0:
+         self.mp.tree_pb.set_fraction(self.done / self.total)
+      return retval
+   def __init__(self, mp, c, total, start_time):
+      self.mp = mp
+      self.c = c
+      self.total = float(total)
+      self.start_time = start_time
+      self.done = 0.0
+      self.d = 0
+      self.art = self.alb = ""
+
 class MediaPane(gtk.Frame):
    """ UI for viewing the prokyon 3 database"""
    
@@ -165,48 +222,33 @@ class MediaPane(gtk.Frame):
       self.db = db
       self.whereentry.set_text("")
       self.fuzzyentry.set_text("")
-      self.tree_update.clicked()
       self.show()
+      self.tree_update.clicked()
    
    def deactivate(self):
       self.hide()
-      del self.db
+      try:
+         del self.db
+      except AttributeError:
+         pass
 
    def cb_tree_update(self, widget):
       c = self.db.cursor()
       try:
-         c.execute("""SELECT id,artist,album,tracknumber,title,length,bitrate,filename,path FROM tracks ORDER BY artist,album,path,tracknumber,title""")
+         total = c.execute("""SELECT id,artist,album,tracknumber,title,length,bitrate,filename,path FROM tracks ORDER BY artist,album,path,tracknumber,title""")
       except dberror.MySQLError, inst:
          print inst
+         c.close()
       else:
          self.treeview.set_model(None)
          self.treestore.clear()
-         art = alb = ""
-         d = 0
-         while 1:
-            row = c.fetchone()
-            if row is None:
-               break
-            while 1:
-               if d == 0:
-                  art = row[1]
-                  iter1 = self.treestore.append(None, (-1, art, 0, 0, 0, "", "")) 
-                  d = 1
-               if d == 1:
-                  if art.lower() == row[1].lower():
-                     alb = row[2]
-                     iter2 = self.treestore.append(iter1, (-2, alb, 0, 0, 0, "", "")) 
-                     d = 2
-                  else:
-                     d = 0
-               if d == 2:
-                  if art.lower() == row[1].lower() and alb.lower() == row[2].lower():
-                     self.treestore.append(iter2, (row[0], row[4], row[3], row[5], row[6], row[7], row[8]))
-                     break
-                  else:
-                     d = 1
-         self.treeview.set_model(self.treestore)
-      c.close()
+         self.treescroll.hide()
+         self.treealt.show()
+         self.tree_pb.set_fraction(0.0)
+         if self.tree_idle is not None:
+            gobject.source_remove(self.tree_idle)
+         tree_populate = TreePopulate(self, c, total, time.time())
+         self.tree_idle = gobject.idle_add(tree_populate.run)
       
    def cb_update(self, widget):
       """ Database lookup performed here """
@@ -302,7 +344,7 @@ class MediaPane(gtk.Frame):
       
       buttonbox.show()
       
-      self.treeview = makeview(self.notebook, ln.media_tree, buttonbox)
+      self.treeview, self.treescroll, self.treealt = makeview(self.notebook, ln.media_tree, buttonbox)
       self.treeview.set_enable_tree_lines(True)
       self.treeview.set_rubber_banding(True)
       treeselection = self.treeview.get_selection()
@@ -326,6 +368,17 @@ class MediaPane(gtk.Frame):
       self.treeview.connect_after("drag-begin", self.cb_drag_begin)
       self.treeview.connect("drag_data_get", self.cb_tree_drag_data_get)
       
+      vbox = gtk.VBox()
+      vbox.set_border_width(20)
+      vbox.set_spacing(20)
+      label = gtk.Label(ln.populating)
+      vbox.pack_start(label, False, False, 0)
+      self.tree_pb = gtk.ProgressBar()
+      vbox.pack_start(self.tree_pb, False, False, 0)
+      self.treealt.add(vbox)
+      vbox.show_all()
+      
+      self.tree_idle = None
       
       # flat gui
       filterframe = gtk.Frame(ln.dbfilters)
@@ -368,7 +421,7 @@ class MediaPane(gtk.Frame):
       wherehbox.pack_start(self.update, False, False, 0)
       self.update.show()
       
-      self.flatview = makeview(self.notebook, ln.media_flat, filterframe)
+      self.flatview, self.flatscroll, self.flatalt = makeview(self.notebook, ln.media_flat, filterframe)
       self.flatview.set_rules_hint(True)
       self.flatview.set_rubber_banding(True)
       treeselection = self.flatview.get_selection()
@@ -387,9 +440,6 @@ class MediaPane(gtk.Frame):
             (ln.dbfilename, 8, None, 100),
             (ln.dbpath, 9, None, -1),
             ))
-      #for ix, col in enumerate(self.flatcols):
-      #   col.set_sort_column_id((0,2,3,4,5,6,7,8,9)[ix])
-      #   col.set_clickable(False)
          
       self.flatview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, self.sourcetargets, gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY)
       self.flatview.connect_after("drag-begin", self.cb_drag_begin)
@@ -442,7 +492,7 @@ class Prefs(gtk.Frame):
          else:
             try:
                self.db.close()
-            except (NameError, dberror.MySQLError):
+            except (AttributeError, NameError, dberror.MySQLError):
                pass
             else:
                self.set_ui_state(False)
@@ -510,6 +560,7 @@ class Prefs(gtk.Frame):
       table.attach(passwordlabel, 0, 1, 1, 2, gtk.SHRINK | gtk.FILL)
       passwordlabel.show()
       self.prokpassword = self.ProkEntry()
+      self.prokpassword.set_visibility(False)
       table.attach(self.prokpassword, 1, 2, 1, 2)
       self.prokpassword.show()
       self.proktoggle = gtk.ToggleButton(ln.prokyon3_connect)
